﻿using System;
using System.Drawing;
using System.Collections.Generic;
using System.Net.Sockets;
using System.Windows.Forms;

namespace FadeCandyTest {
	public partial class Form1: Form {
		#region Variables
		private const String IP = "127.0.0.1";
		private const Int32 PORT = 7890;

		private TcpClient _client;
		private NetworkStream _ns;
		private String _log = "";

		private Bitmap _bmp;
		private Int32 _timerTick;
		#endregion

		#region Ctor
		public Form1() {
			InitializeComponent();
		}
		#endregion

		#region Util
		private void _sendCommandPacket(Byte[] buffer) {
			String errorText = this._open();
			if (!String.IsNullOrEmpty(errorText)) { this.txtLog.Text = errorText; return; }
			this._write(buffer, ref errorText);
			if (!String.IsNullOrEmpty(errorText)) { this.txtLog.Text = errorText; return; }
		}

		private String _open() {
			if (this._client == null || !this._client.Connected) {
				this._client = new TcpClient();
				this._client.NoDelay = true;
				this._client.SendBufferSize = 8000;
				this._client.SendTimeout = 100;
				this._client.ReceiveBufferSize = 8000;
				this._client.ReceiveTimeout = 100;
				this._client.Client.DontFragment = true;
				this._client.ExclusiveAddressUse = true;
				this._client.LingerState = new LingerOption(true, 300);

				try {
					this._client.Connect(IP, PORT);
					this._ns = this._client.GetStream();
					return "";
				}
				catch (Exception ex) {
					this._client = null;
					return ex.Message;
				}
			}
			else {
				return "";
			}
		}

		public void _write(Byte[] btWrite, ref String errorText) {
			try {
				if (btWrite.Length > 0) {
					this._ns.Write(btWrite, 0, btWrite.Length);
					this._ns.Flush();
				}
				errorText = "";
			}
			catch (Exception ex) { errorText = ex.Message; }
		}

		private String _close() {
			try {
				this.timer1.Enabled = false;
				if (this._client != null) { this._client.Close(); }
				return "";
			}
			catch (Exception ex) { return ex.Message; }
			finally {
				this._ns = null;
				this._client = null;
			}
		}

		private void _setPixel(Byte r, Byte g, Byte b) {
			for (Int32 i = 1; i <= 2; i++) {
				this._sendCommandPacket(new Byte[] { 
				0x00, 0x00, /* канал=0x00, команда=0x00 */
				0x00, 0x06, /* длина данных=6 */
				r, g, b /* R0, G0, B0 */,
				r, g, b /* R1, G1, B1 */});
			}
		}

		/// <summary>
		/// Рисуем картинку
		/// </summary>
		/// <param name="image"></param>
		/// <param name="r"></param>
		/// <param name="g"></param>
		/// <param name="b"></param>
		private void _setPixels(String image, Byte r, Byte g, Byte b) {
			Byte[] packet = new Byte[4 + image.Length * 3];
			// Пусть image.Length = 64 (по символу на каждый пиксель)
			packet[3] = 192; // длина данных=3*64=192
			for (Int32 i = 0; i < image.Length; i++) {
				if (image.Substring(i, 1) == " ") {
					// Не светим, т.е. 0,0,0
				}
				else {
					// Светим
					packet[4 + i * 3] = r;
					packet[5 + i * 3] = g;
					packet[6 + i * 3] = b;
				}
			}

			for (Int32 i = 1; i <= 2; i++) {
				this._sendCommandPacket(packet);
			}
		}

		/// <summary>
		/// Отсылаем массив пикселей
		/// </summary>
		/// <param name="bt"></param>
		private void _setPixels(Byte[] bt) {
			Byte[] packet = new Byte[4 + bt.Length];
			packet[3] = (Byte) bt.Length; // длина данных
			for (Int32 i = 0; i < bt.Length; i++) {
				packet[4 + i] = bt[i];
			}

			for (Int32 i = 1; i <= 2; i++) {
				this._sendCommandPacket(packet);
			}
		}
		#endregion

		#region Control events
		private void cmdSetRed0_Click(object sender, EventArgs e) {
			this._setPixel(127, 0, 0);
		}
		
		private void cmdSetGreen0_Click(object sender, EventArgs e) {
			this._setPixel(0, 127, 0);
		}

		private void cmdSetBlue0_Click(object sender, EventArgs e) {
			this._setPixels(
				"FFFFFFFF" +
				"FFFFFFFF" +
				"FF      " +
				"FFFFF   " +
				"FFFFF   " +
				"FF      " +
				"FF      " +
				"FF      ", 64, 0, 0);
		}
						
		private void timer1_Tick(object sender, EventArgs e) {
			this.timer1.Enabled = false;
			Byte[] bt = new Byte[64 * 3];
			Int32 j = 0;
			for (Int32 y = 0; y < 8; y++) {
				for (Int32 x = this._timerTick; x < this._timerTick + 8; x++) {
					Color c = this._bmp.GetPixel(x, y);
					bt[j] = c.R;
					j++;
					bt[j] = c.G;
					j++;
					bt[j] = c.B;
					j++;
				}
			}
			// Ограничим яркость до 64
			for (Int32 i = 0; i < bt.Length; i++) {
				if (bt[i] > 64) { bt[i] = 64; }
			}

			this._setPixels(bt);
			this._timerTick++;
			if (this._timerTick < this._bmp.Width - 8)
				this.timer1.Enabled = true;
		}
		#endregion

		private void cmdShow_Click(object sender, EventArgs e) {
			this._showText("Hello cruel world!");
			this._timerTick = 0;
			this.timer1.Interval = 100;
			this.timer1.Enabled = true;
		}

		private void _showText(String text) {
			this._bmp = new Bitmap(500, 8, System.Drawing.Imaging.PixelFormat.Format24bppRgb);
			using (Graphics g = Graphics.FromImage(this._bmp)) {
				g.Clear(Color.Black);
				g.DrawString(text, new Font("arial", 6), new SolidBrush(Color.Red), 
					new PointF(-2, -3));
			}
			this._bmp.Save("1.bmp", System.Drawing.Imaging.ImageFormat.Png);
		}
	}
}